*! version 1.1.2  02JUL2010
capture program drop malmq
program define malmq, rclass
    version 10.0

// syntax checking and validation-----------------------------------------------
// rts - return to scale, ort - orientation
// input varlist = output varlist
// -----------------------------------------------------------------------------
    // returns 1 if the first nonblank character of local macro `0' is a comma,
    // or if `0' is empty.
    if replay() {
        dis as err "ivars and ovars must be inputed."
        exit 198
    }

    // get and check invarnames
    gettoken word 0 : 0, parse(" =:,")
    while `"`word'"' != ":" & `"`word'"' != "=" {
        if `"`word'"' == "," | `"`word'"'=="" {
                error 198
        }
        local invarnames `invarnames' `word'
        gettoken word 0 : 0, parse(" :,")
    }
    unab invarnames : `invarnames'

    #del ;
    syntax varlist(min=1) [if] [in] [using/]
    [,
        Period(name)
        ORT(string)       // ignore case sensitive,[{IN|INPUT}|{OUT|OUTPUT}]
        TOL1(real 1e-14)  // entering or leaving value tolerance
        TOL2(real 2.22e-12) // B inverse tolerance
        MINSUBScript      // minimal subscript pivot
        TRACE             // Whether or not to do the log
        SAVing(string)    // log file name
        REPLACE           // Whether or not to replace the log file
    ];
    #del cr

    local num_invar : word count `invarnames'
    local i 1
    while (`i'<=`num_invar') {
        local invarn : word `i' of `invarnames'
        local junk : subinstr local invarnames "`invarn'" "", ///
            word all count(local j)
        if `j' > 1 {
            di as error ///
                "cannot specify the same input variable more than once"
            error 498
        }
        local i = `i' + 1
    }

    // default orientation - Input Oriented
    if ("`ort'" == "") local ort = "IN"
    else {
        local ort = upper("`ort'")
        if ("`ort'" == "I" | "`ort'" == "IN" | "`ort'" == "INPUT") {
            local ort = "IN"
        }
        else if ("`ort'" == "O" | "`ort'" == "OUT" | "`ort'" == "OUTPUT") {
            local ort = "OUT"
        }
        else {
            di as err "option ort allow for case-insensitive " _c
            di as err "(i|in|input|o|out|output) or nothing."
            exit 198
        }
    }

    if ("`using'" != "") use "`using'", clear
    if (~(`c(N)' > 0 & `c(k)' > 0)) {
        dis as err "dataset is not ready!"
        exit 198
    }

    if ("`period'" == "") {
        di as err "you must specify period (varname)"
        exit 198
    }

// end of syntax checking and validation ---------------------------------------

    set more off
    capture log close malmquistlog
    log using "malmquist.log", replace text name(malmquistlog)
    preserve

    if ("`if'" != "" | "`in'" != "") {
        qui keep `in' `if'  // filtering : keep in range [if exp]
    }

    deamalmquist `period', ivars(`invarnames') ovars(`varlist') ///
                  ort(`ort') tol1(`tol1') tol2(`tol2') `minsubscript' ///
                  `trace' saving(`saving') `replace'
    return add

    restore, preserve
    log close malmquistlog
end

********************************************************************************
* DEA Malmquist
********************************************************************************
program define deamalmquist, rclass
    #del ;
    syntax varname(numeric), IVARS(string) OVARS(string) ORT(string)
        [
            TOL1(real 1e-3) TOL2(real 1e-12) MINSUBScript
            TRACE SAVing(string) REPLACE
        ];
    #del cr
    preserve

    // di _n(2) as input "options: ts(`varlist') ORT(`ort')"
    // di as input "Input Data:"
    // list

    tempname tsvec
    local tsn = "`varlist'"
    mata: _uniqrowmat("`tsvec'", "`tsn'") // sorted unique row matrix
    if (rowsof(`tsvec') < 2) {
        di as err "time series `tsn' observations must be ge 2"
        exit 451 // invalid values for time variable
    }

    // crs{i|o|f|r}: crs {input|output|frame|result}
    // t, t1: t+1, c: cross --> crsi_ct1: crsi cross t+1,
    tempname crsi_t crso_t crsf_t crsr_t crsi_t1 crso_t1 crsf_t1 crsr_t1
    tempname crsr_ct crsr_ct1
    tempname vrsi_t vrso_t vrsf_t vrsr_t vrsi_t1 vrso_t1 vrsf_t1 vrsr_t1
    tempname crossrslt effr_t1 effrslt prodidxr_t prodidxrslt
    tempname tfpch effch techch pech sech
    local stage = 1
    forvalues t = 1/`=rowsof(`tsvec') - 1' {
        local tsval_t = `tsvec'[`t',1]
        local tsval_t1 = `tsvec'[`t'+1,1]

        // CRS-DEA t
        local rts = "CRS"
        if (`t' == 1) {
            if ("`trace'" != "") di _newline "CRS-DEA ts[`tsval_t']:"
            mkDmuMat `ivars' if `tsn'==`tsval_t', dmumat(`crsi_t') sprefix("i")
            mkDmuMat `ovars' if `tsn'==`tsval_t', dmumat(`crso_t') sprefix("o")
            mata: _mkframemat("`crsf_t'", "`crsi_t'", "`crso_t'", ///
                    "`rts'", "`ort'", `stage')
            deamain `crsi_t' `crso_t' `crsf_t' `rts' `ort' `stage' ///
                    `tol1' `tol2' `minsubscript' `trace'
            matrix `crsr_t' = r(deamainrslt)
        }
        else {
            matrix `crsi_t' = `crsi_t1'
            matrix `crso_t' = `crso_t1'
            matrix `crsf_t' = `crsf_t1'
            matrix `crsr_t' = `crsr_t1'
        }
        // CRS-DEA t+1
        if ("`trace'" != "") di _newline "CRS-DEA ts[`tsval_t1']:"
        mkDmuMat `ivars' if `tsn'==`tsval_t1', dmumat(`crsi_t1') sprefix("i")
        mkDmuMat `ovars' if `tsn'==`tsval_t1', dmumat(`crso_t1') sprefix("o")
        mata: _mkframemat("`crsf_t1'", "`crsi_t1'", "`crso_t1'", ///
                    "`rts'", "`ort'", `stage')
        deamain `crsi_t1' `crso_t1' `crsf_t1' `rts' `ort' `stage' ///
                `tol1' `tol2' `minsubscript' `trace'
        matrix `crsr_t1' = r(deamainrslt)

        // Cross CRS-DEA t
        if ("`trace'" != "") di _newline "Cross CRS-DEA t[`tsval_t']:"
        deamain `crsi_t1' `crso_t1' `crsf_t' `rts' `ort' `stage' ///
                `tol1' `tol2' `minsubscript' `trace'
        matrix `crsr_ct' = r(deamainrslt)
        // Cross CRS-DEA t+1
        if ("`trace'" != "") di _newline "Cross CRS-DEA t+1[`tsval_t1']:"
        deamain `crsi_t' `crso_t' `crsf_t1' `rts' `ort' `stage' ///
                `tol1' `tol2' `minsubscript' `trace'
        matrix `crsr_ct1' = r(deamainrslt)

        // VRS-DEA t
        local rts = "VRS"
        if (`t' == 1) {
            if ("`trace'" != "") di _newline "VRS-DEA ts[`tsval_t']:"
            mkDmuMat `ivars' if `tsn'==`tsval_t', dmumat(`vrsi_t') sprefix("i")
            mkDmuMat `ovars' if `tsn'==`tsval_t', dmumat(`vrso_t') sprefix("o")
            mata: _mkframemat("`vrsf_t'", "`vrsi_t'", "`vrso_t'", ///
                    "`rts'", "`ort'", `stage')
            deamain `vrsi_t' `vrso_t' `vrsf_t' `rts' `ort' `stage' ///
                    `tol1' `tol2' `minsubscript' `trace'
            matrix `vrsr_t' = r(deamainrslt)
        }
        else {
            matrix `vrsi_t' = `vrsi_t1'
            matrix `vrso_t' = `vrso_t1'
            matrix `vrsf_t' = `vrsf_t1'
            matrix `vrsr_t' = `vrsr_t1'
        }
        // VRS-DEA t+1
        if ("`trace'" != "") di _newline "VRS-DEA ts[`tsval_t1']:"
        mkDmuMat `ivars' if `tsn'==`tsval_t1', dmumat(`vrsi_t1') sprefix("i")
        mkDmuMat `ovars' if `tsn'==`tsval_t1', dmumat(`vrso_t1') sprefix("o")
        mata: _mkframemat("`vrsf_t1'", "`vrsi_t1'", "`vrso_t1'", ///
                    "`rts'", "`ort'", `stage')
        deamain `vrsi_t1' `vrso_t1' `vrsf_t1' `rts' `ort' `stage' ///
                `tol1' `tol2' `minsubscript' `trace'
        matrix `vrsr_t1' = r(deamainrslt)

        // ------------
        // make Malmquist efficiency DEA Results
        if (`t' == 1) {
            local dmuCount = colsof(`crsi_t')
            local rnames : colfullnames `crsi_t'
            local cnames = "`tsn' CRS_eff VRS_eff"

            matrix `effrslt' = J(`dmuCount', 1, `tsval_t')
            matrix `effrslt' = `effrslt', `crsr_t'[1...,1], `vrsr_t'[1...,1]
            matrix rownames `effrslt' = `rnames'
            matrix colnames `effrslt' = `cnames'
        }
        matrix `effr_t1' = J(`dmuCount', 1, `tsval_t1')
        matrix `effr_t1' = `effr_t1', `crsr_t1'[1...,1], `vrsr_t1'[1...,1]
        matrix rownames `effr_t1' = `rnames'

        matrix `effrslt' = `effrslt' \ `effr_t1'

        // make Malmquist productivity index DEA Results
        // tfpch: total factor productivity change
        // effch: techinical efficiency change
        // techch: technological change
        // pech: pure techinical efficiency change
        // sech: scale efficiency change
        matrix `prodidxr_t' = J(`dmuCount', 1, `tsval_t')
        matrix `prodidxr_t' = `prodidxr_t', J(`dmuCount', 1, `tsval_t1')
        matrix `prodidxr_t' = `prodidxr_t', J(`dmuCount', 5, 0)
        forvalues dmui = 1/`dmuCount' {
            scalar `tfpch' = sqrt((`crsr_ct'[`dmui',1] ///
                                 / `crsr_t'[`dmui',1]) ///
                                * (`crsr_t1'[`dmui',1] ///
                                 / `crsr_ct1'[`dmui',1]))
            scalar `effch' = `crsr_t1'[`dmui',1]/`crsr_t'[`dmui',1]
            scalar `techch' = sqrt((`crsr_ct'[`dmui',1] ///
                                  / `crsr_t1'[`dmui',1]) ///
                                 * (`crsr_t'[`dmui',1] ///
                                  / `crsr_ct1'[`dmui',1]))
            scalar `pech' = `vrsr_t1'[`dmui',1]/`vrsr_t'[`dmui',1]
            scalar `sech' = `effch'/`pech'
            matrix `prodidxr_t'[`dmui', 3] = ///
                (`tfpch', `effch', `techch', `pech', `sech')
        }
        matrix rownames `prodidxr_t' = `rnames'
        if (`t' == 1) matrix colnames ///
            `prodidxr_t' = from thru tfpch effch techch pech sech

        matrix `prodidxrslt' = nullmat(`prodidxrslt') \ `prodidxr_t'

        matrix `crossrslt' = nullmat(`crossrslt') \ ///
                (`prodidxr_t'[1...,1..2], `crsr_ct'[1...,1], `crsr_ct1'[1...,1])
    }

    // -------------------------------------------------------------------------
    // REPORT
    // -------------------------------------------------------------------------
    di _n(2) as txt "Cross CRS-DEA Result:"
    matrix colnames `crossrslt' = from thru t t1
    matrix rownames `crossrslt' = `: rowfullnames `prodidxrslt''
    matrix list `crossrslt', noblank nohalf noheader f(%9.6g)

    di _n "Malmquist efficiency `ort'PUT Oriented DEA Results:"
    // matrix list `effrslt', noblank nohalf noheader f(%9.6g)
    qui {
        keep dmu
        gen str dmu1 = dmu
        rename dmu `tsn'1
        rename dmu1 dmu
        svmat `effrslt', names(col)
        replace `tsn'1 = string(`tsn')
        drop `tsn'
        rename `tsn'1 `tsn'
        format CRS_eff VRS_eff %9.6g
    }
    list
    return matrix effrslt = `effrslt'

    di _n "Malmquist productivity index `ort'PUT Oriented DEA Results:"
    // matrix list `prodidxrslt', noblank nohalf noheader f(%9.6g)
    restore, preserve
    qui {
        keep dmu
        keep in 1/`=rowsof(`prodidxrslt')'
        gen str dmu1 = dmu
        rename dmu period
        rename dmu1 dmu
        svmat `prodidxrslt', names(col)
        replace period = string(from) + "~" + string(thru)
        drop from thru
        format tfpch effch techch pech sech %9.6g
    }
    list
    return matrix prodidxrslt = `prodidxrslt'

    if ("`saving'"!="") {
        // if save file exist and don't replace, make the backup file.
        if ("`replace'" == "") {
            local dotpos = strpos("`saving'",".")
            if (`dotpos' > 0) {
                mata: file_exists("`saving'")
            }
            else {
                mata: file_exists("`saving'.dta")
            }
            if r(fileexists) {
                local curdt = subinstr("`c(current_date)'", " ", "", .) + /*
                    */ subinstr("`c(current_time)'", ":", "", .)
                if (`dotpos' > 0) {
                    #del ;
                    local savefn = substr("`saving'", 1, `dotpos' - 1) +
                                   "_bak_`curdt'" +
                                   substr("`saving'",`dotpos', .);
                    #del cr
                    qui copy "`saving'" "`savefn'", replace
                }
                else {
                    local savefn = "`saving'_bak_`curdt'" + ".dta"
                    qui copy "`saving'.dta" "`savefn'", replace
                }
            }
        }
        capture save `saving', replace
    }
    restore, preserve
end

********************************************************************************
* DEA Main - Data Envelopment Analysis Main
********************************************************************************
program define deamain, rclass
    args dmuIn dmuOut frameMat rts ort stage tol1 tol2 minsubscript trace

    tempname efficientVec deamainrslt

    // stage step 1.
    if ("`trace'" == "trace") {
        di _n(2) as txt "RTS(`rts') ORT(`ort') 1st stage."
    }
    mata: _dealp("`frameMat'", "`dmuIn'", "`dmuOut'", "`rts'", "`ort'", ///
        1, `tol1', `tol2', "`minsubscript'", "`efficientVec'", "`trace'")
    matrix `deamainrslt' = r(dealprslt)

    // stage step 2.
    if ("`stage'" == "2") {
        if ("`trace'" == "trace") {
            di _n(2) as txt "RTS(`rts') ORT(`ort') 2nd stage."
        }
        matrix `efficientVec' = `deamainrslt'[1...,1]

        mata: _dealp("`frameMat'", "`dmuIn'", "`dmuOut'", "`rts'", "`ort'", ///
            2, `tol1', `tol2', "`minsubscript'", "`efficientVec'", "`trace'")
        matrix `deamainrslt' = r(dealprslt)
    }

    // // if output oriented, get theta from eta
    // if ("`ort'" == "OUT") {
    //     tempname eta
    //     forvalues i = 1/`=rowsof(`deamainrslt')' {
    //         scalar `eta' = el(`deamainrslt', `i', 1)
    //         matrix `deamainrslt'[`i',1] = 1/`eta'
    //         matrix `deamainrslt'[`i',2] = `deamainrslt'[`i',2...]/`eta'
    //     }
    // }

    // adjust negative value
    forvalues i = 1/`=rowsof(`deamainrslt')' {
        forvalues j = 1/`=colsof(`deamainrslt')' {
            if (`deamainrslt'[`i',`j'] < 0) {
                matrix `deamainrslt'[`i',`j'] = 0
            }
        }
    }

    return matrix deamainrslt = `deamainrslt'
end

********************************************************************************
* Data Import and Conversion
********************************************************************************

// Make DMU Matrix -------------------------------------------------------------
program define mkDmuMat
    #del ;
    syntax varlist(numeric) [if] [in], DMUmat(name)
    [
        SPREfix(string)
    ];
    #del cr

    qui ds
    // variable not found error
    if ("`varlist'" == "") {
        di as err "variable not found"
        exit 111
    }

    // make matrix
    mkmat `varlist' `if' `in', matrix(`dmumat') rownames(dmu)
    matrix roweq `dmumat' = "dmu"
    matrix coleq `dmumat' = `=lower("`sprefix'") + "slack"'
    matrix `dmumat' = `dmumat''
end

// Start of the MATA Definition Area -------------------------------------------

version 10
mata:
mata set matastrict on

/**
 * make frame matrix and set matrix value at the param frameMat
 * rts - return to scale, ort - orientation
 */
function _mkframemat( string scalar frameMat,
                      string scalar dmuIn,
                      string scalar dmuOut,
                      string scalar rts,
                      string scalar ort,
                      real scalar stage )
{
    real matrix F, DI, DO
    real scalar row, col, sig, isin, isout
    real scalar dmus, slackins, slackouts, slacks, artificials, totalSlacks
    real scalar frows, fcols

    DI = st_matrix(dmuIn)
    DO = st_matrix(dmuOut)
    if (cols(DI) != cols(DO)) {
        _error(3200, "in and out count of dmu is not match!")
    }

    // basic value setting for artificial variabels
    isin = (ort == "IN"); isout = !isin
    sig = (isin ? -1 : 1)

    dmus = cols(DI) // or cols(DO), because cols(DI) == cols(DO)
    slackins = rows(DI); slackouts = rows(DO)
    if (rts == "CRS") {
        slacks = slackins + slackouts
        artificials = ((isout && stage == 1) ? 0 : slacks)
        frows = 1 + slacks
        fcols = 3 + dmus + slacks + artificials
    }
    else if (rts == "VRS") {
        slacks = slackins + slackouts + 1
        artificials = slacks
        frows = 1 + slacks
        fcols = 3 + dmus + slacks + artificials
    }
    else if (rts == "DRS") {
        slacks = slackins + slackouts + 1
        artificials = ((isout && stage == 1) ? 0 : slacks)
        frows = 1 + slacks
        fcols = 3 + dmus + slacks + artificials
    }
    else if (rts == "IRS") {
        slacks = slackins + slackouts + 1
        artificials = slacks
        frows = 1 + slacks
        fcols = 3 + dmus + slacks + artificials
    }
    else {
        _error(3498, "invalid rts optoins")
    }

    // make frame matrix for CRS(CCR)
    F = J(frows, fcols, 0)
    F[1, 1] = 1
    replacesubmat(F, 2, 3, sig * DI)
    replacesubmat(F, 2 + slackins, 3, -sig * DO)
    replacesubmat(F, 2, 3 + dmus, sig * I(slackins + slackouts))
    if (artificials > 0) {
        replacesubmat(F, 1, fcols - artificials, J(1,artificials,-1))
        replacesubmat(F, 2, fcols - artificials, I(artificials))
    }

    // adjustment
    if (rts == "CRS") {
    }
    else if (rts == "VRS") {
        replacesubmat(F, frows, 3, J(1, dmus, 1))
        F[frows,fcols] = 1
    }
    else if (rts == "DRS") {
        replacesubmat(F, frows, 3, J(1, dmus, 1))
        F[frows,2 + dmus + slacks] = 1
        F[frows,fcols] = 1
    }
    else if (rts == "IRS") {
        replacesubmat(F, frows, 3, J(1, dmus, 1))
        F[frows,2 + dmus + slacks] = -1
        F[frows,fcols] = 1
    }

    // return result
    st_matrix(frameMat, F)
}

/**
 * DEA Loop - Data Envelopment Analysis Loop for DMUs
 */
function _dealp ( string scalar frameMat,
                  string scalar dmuIn,
                  string scalar dmuOut,
                  string scalar rts,
                  string scalar ort,
                  real scalar stagestep,
                  real scalar tol1,
                  real scalar tol2,
                  string scalar minsubscript,
                  string scalar efficientVec,
                  string scalar trace )
{
    real matrix F, DI, DO, M, VARS, LPRSLT, DEALPRSLT
    real scalar dmus, slackins, slackouts, slacks, artificials
    real scalar frows, fcols, isin, isout, isminsubscript, skipPhase1
    real colvector effvec
    string scalar tracename

    F  = st_matrix(frameMat)
    DI = st_matrix(dmuIn)
    DO = st_matrix(dmuOut)
    if (cols(DI) != cols(DO)) {
        _error(3200, "in and out count of dmu is not match!")
    }

    // basic value setting for artificial variabels
    isin = (ort == "IN"); isout = !isin
    isminsubscript = (minsubscript != "nominsubscript")
    frows = rows(F); fcols = cols(F)
    dmus = cols(DI) // or cols(DO), because cols(DI) == cols(DO)
    slackins = rows(DI); slackouts = rows(DO)
    slacks = frows - 1 // slacks is not equals to (slackins + slackouts)
    artificials = fcols - (3 + dmus + slacks)

    tracename = rts + "-" + (isin ? "IN" : "OUT")
    tracename = tracename + "-" + (stagestep == 1 ? "SI" : "SII")

    DEALPRSLT = J(0, 1+ dmus + slacks, .)
    if (isin) {
        if (stagestep == 2) {
            effvec = st_matrix(efficientVec)
            replacesubmat(F, 2, 3, -F[2..1+slackins,3::2+dmus+slackins])
        }

        for (dmui=1; dmui<=dmus; dmui++) {
            M = F
            if (stagestep == 1) {
                replacesubmat(M, 2, 2, DI[,dmui])
            }
            else {
                replacesubmat(M, 2, fcols, DI[,dmui] * effvec[dmui])
            }
            replacesubmat(M, 2+slackins, fcols, DO[,dmui])

            // LP execute...
            VARS = lp_phase1(M, dmus, dmui, rts, stagestep,
                    isin, tol1, tol2, isminsubscript, tracename, trace)
            LPRSLT = lp_phase2(M, VARS, dmus, dmui, rts, stagestep,
                    isin, tol1, tol2, isminsubscript, tracename, trace);
            DEALPRSLT = DEALPRSLT \ LPRSLT
        }
    }
    else {
        if (stagestep == 2) {
            effvec = st_matrix(efficientVec)
            replacesubmat(F, 2+slackins, 3,
                -F[2+slackins..1+slackins+slackouts,3::2+dmus+slacks])
        }

        skipPhase1 = (stagestep == 1 && (rts == "CRS" || rts == "DRS"))
        for (dmui=1; dmui<=dmus; dmui++) {
            M = F
            replacesubmat(M, 2, fcols, DI[,dmui])
            if (stagestep == 1) {
                if (rts == "CRS" || rts == "DRS") M[1,2] = -1
                replacesubmat(M, 2+slackins, 2, DO[,dmui])
            }
            else {
                replacesubmat(M, 2+slackins, fcols, DO[,dmui] * effvec[dmui])
            }

            // LP execute...
            if (skipPhase1) {
                VARS   = (0, 2+dmus..1+dmus+slacks, 1..1+dmus, 0)
                M = M[,1], M[,VARS[,2::cols(VARS)-1] :+ 1], M[,cols(M)]
            }
            else {
                VARS = lp_phase1(M, dmus, dmui, rts, stagestep,
                    isin, tol1, tol2, isminsubscript, tracename, trace)
            }
            LPRSLT = lp_phase2(M, VARS, dmus, dmui, rts, stagestep,
                    isin, tol1, tol2, isminsubscript, tracename, trace)
            DEALPRSLT = DEALPRSLT \ LPRSLT
        }
    }

    // adjust efficiency
    if (stagestep == 2) {
        replacesubmat(DEALPRSLT, 1, 1, effvec)
    }
    st_matrix("r(dealprslt)", DEALPRSLT)
}

real matrix function lp_phase1 ( real matrix M,
                                 real scalar dmus,
                                 real scalar dmui,
                                 string scalar rts,
                                 real scalar stagestep,
                                 real scalar isin,
                                 real scalar tol1,
                                 real scalar tol2,
                                 real scalar isminsubscript,
                                 string scalar aTracename,
                                 string scalar trace )
{
    real matrix T, XB, B, Bi, VARS
    real scalar mrows, mcols, slacks, artificials, artificialval
    string scalar msg

    mrows = rows(M); mcols = cols(M)
    slacks = mrows - 1
    artificials = mcols - (3 + dmus + slacks)
    artificialval = M[1, mcols - 1]

    tracename = aTracename + "-PI"

    // 1st: initialize matrix.
    if (trace == "trace") {
        displayas("txt")
        printf("\n\n\n----------[PHASE I]----------")
        printf("\n[DMUi=%g]%s: initialize matrix.\n",
            dmui, tracename); M
    }

    // 2nd: classify basic and nonbasic.
    T = M
    replacesubmat(M, 1, 2, T[,3+dmus+slacks::2+dmus+slacks+artificials])
    replacesubmat(M, 1, 2+slacks, T[,2::2+dmus+slacks])

    if (trace == "trace") {
        displayas("txt")
        printf("\n[DMUi=%g]%s: classify basic and nonbasic.\n",
            dmui, tracename); M
    }

    // 3rd: slove the lenear problem(LP).
    VARS = (0, -1..-artificials, 1..1+dmus+slacks, 0); phase = 1
    XB = lp(M, VARS, dmus, dmui, rts, stagestep, phase,
            isin, tol1, tol2, isminsubscript, tracename, trace)

    // -------------------------------------------------------------------------
    // phase 1 final.
    // 1. remove the artificial.
    T = 1..mrows
    for (j=1+mrows ; j<=mcols ; j++) {
        if (VARS[1,j] >= 0) T = T, j
    }
    M = M[,T]; VARS = VARS[,T]

    // 2. setting the target function value of theta
    T = VARS[1,2::1+slacks]; T = T :/ (T :> 0)
    minindex(T, 1, mi, w); M[1,mi[1]+1] = -1
    return(VARS)
}

real matrix function lp_phase2 ( real matrix M,
                                 real matrix VARS,
                                 real scalar dmus,
                                 real scalar dmui,
                                 string scalar rts,
                                 real scalar stagestep,
                                 real scalar isin,
                                 real scalar tol1,
                                 real scalar tol2,
                                 real scalar isminsubscript,
                                 string scalar aTracename,
                                 string scalar trace )
{
    real matrix T, XB, orgVARS, LPRSLT
    real scalar mrows, mcols, slacks, artificials
    string scalar msg, tracename

    orgVARS = VARS
    mrows = rows(M); mcols = cols(M)
    slacks = mrows -1
    artificials = mcols - (3 + dmus + slacks)

    tracename = aTracename + "-PII"

    if (trace == "trace") {
        displayas("txt")
        printf("\n----------[PHASE II]----------")
        printf("\n[DMUi=%g]%s: initialize matrix.\n",
            dmui, tracename); M
        printf("\n[DMUi=%g]%s: VARS.\n", dmui, tracename); VARS
    }

    phase = 2
    XB = lp(M, VARS, dmus, dmui, rts, stagestep, phase,
            isin, tol1, tol2, isminsubscript, tracename, trace)

    // -------------------------------------------------------------------------
    // phase 2 final.
    // -------------------------------------------------------------------------
    LPRSLT = J(1, 1+dmus+slacks, .) // lpresult = theta(1) + dmus + slacks
    for (j=1 ; j<=rows(XB) ; j++) {
        if (VARS[1,j+1] > 0) LPRSLT[1, VARS[1,j+1]] = XB[j, 1]
    }

    if (stagestep == 1 && LPRSLT[1, 1] >= .) {
        displayas("txt")
        printf("\n[DMUi=%g]%s: Entered index.\n", dmui, tracename); VARS[1,2::1+rows(XB)]
        printf("\n[DMUi=%g]%s: XB.\n", dmui, tracename); XB

        errmsg = "No Solution at Stage 1 and Phase 2.:"
        errmsg = errmsg + "[DMUi=" + strofreal(dmui) + "]"
        _error(3498, errmsg)
    }

    if (trace == "trace") {
        msg = sprintf("[DMUi=%g]%s-FINAL", dmui, tracename);
        printf("\n%s: original VARS.\n", msg); orgVARS
        printf("\n%s: VARS.\n", msg); VARS
        printf("\n%s: XB.\n", msg); XB
        printf("\n%s: LPRSLT.\n", msg); LPRSLT
    }

    return(LPRSLT)
}

real matrix function lp ( real matrix M,
                          real matrix VARS,
                          real scalar dmus,
                          real scalar dmui,
                          string scalar rts,
                          real scalar stagestep,
                          real scalar phase,
                          real scalar isin,
                          real scalar tol1,
                          real scalar tol2,
                          real scalar isminsubscript,
                          string scalar tracename,
                          string scalar trace )
{
    real matrix B, CB, Bi, b, XB, CBBi, Nj, CNj, Aj, alpha
    real matrix T, LVi, Ei, V, savedM, savedVARS, savedXB
    real scalar mrows, mcols, enteringVar, leavingVar, calci, maxiter, isout
    real scalar slacks, artificials, existArtificial
    real scalar xVal, alphaVal, tempVal, isMAX, isExistSolution

    mrows = rows(M); mcols = cols(M)
    slacks = mrows - 1
    artificials = mcols - (3 + dmus + slacks)
    isout = !isin
    isMAX = (isout && stagestep == 1 && artificials <= 0)
    isExistSolution = 0
    LVi = J(slacks, 1, .) // leaving variable index matrix.

    // -------------------------------------------------------------------------
if (trace == "trace") {
    displayas("txt"); msg = "initial tableau in the LP."
    printf("\n[DMUi=%g]%s: %s\n", dmui, tracename, msg); M
}

    existArtificial = (phase == 2 && any(VARS[,2::1+slacks] :< 0));
    maxiter = st_numscalar("c(maxiter)")
    Bi = edittozerotol(luinv(M[2..mrows,2::1+slacks]), tol2)
    for (calci=1 ; calci<=maxiter ; calci++) { // prevent infinite loop
        if (trace == "trace") {
            printf("\n[DMUi=%g]%s-LOOP[%g]\n", dmui, tracename, calci)
        }

        B   = M[2..mrows,2::1+slacks]
        CB  = M[1,2::1+slacks]
        b   = M[2..mrows,mcols]
        Nj  = M[2..mrows,2+slacks::mcols-1]
        CNj = M[1,2+slacks::mcols-1]

        // BFS(basic feasible solution) must be nonnegative.
        // if bfs is negative, you must use the pervious XB
        T = edittozerotol(Bi * b, tol2); if (any(T :< 0)) break;
        XB   = T
        CBBi = edittozerotol(CB * Bi, tol2)

        // ---------------------------------------------------------------------
        // loop terminated condition.
        // ---------------------------------------------------------------------
        if (phase == 1) {
            xVal = edittozerotol(CBBi * b, tol2)
            T = VARS[1,2::1+rows(XB)]

if (trace == "trace") {
    printf("\n[DMUi=%g]%s-LOOP[%g]: CBBi * b = %g\n",
        dmui, tracename, calci, xVal);
    display("Entered index(if value is negative, that's artificial):"); T
    display("XB = Bi * b:"); XB
}
            // target function value is zero
            // and if stagestep is 2 or theta be pivoted
            // and all artifical value is zero when artificial exist.
            T = (T:< 0)
            if (xVal == 0
                    && (stagestep == 2 || any(LVi :== 1))
                    && (allof(T, 0) || allof(select(XB, T'), 0))) {
                // meet the end requirements
                isExistSolution = 1; if (stagestep == 2) break;

                // for revoke when above condition is false.
                savedM = M
                savedVARS = VARS
                savedXB = XB
            }
            else {
                if (isExistSolution) {
                    M = savedM
                    VARS = savedVARS
                    XB = savedXB
                    break;
                }
            }
        }

        // ---------------------------------------------------------------------
        // Select entering variable.
        // ---------------------------------------------------------------------
        enteringVar = 0; tempVal = 0;
        Njcols = cols(Nj); T = J(1, Njcols, .)
        for (j=1; j<=Njcols; j++) {
            T[1,j] = CBBi * Nj[, j] - CNj[1,j]
            if (abs(T[1,j]) < tol1) continue;

            if ((!isMAX && T[1,j] < 0) || (isMAX && T[1,j] > 0)) {
                if (isminsubscript) {
                    enteringVar = j
                    evj = 1+slacks+enteringVar
                    if (VARS[,evj] > 0) break
                }
                else {
                    if (abs(T[1,j]) > tempVal) {
                        tempVal = abs(T[1,j])
                        enteringVar = j
                        evj = 1+slacks+enteringVar
                    }
                }
            }
        } // end of for

        // No more select entering variable.
        if (enteringVar == 0) {
            if (trace == "trace") {
                printf("\n[DMUi=%g]%s-LOOP[%g]:\n", dmui, tracename, calci)
                display("No more select entering variable."); T
            }
            break
        }

if (trace == "trace") {
    displayas("txt"); msg = "Select entering variable."
    printf("\n[DMUi=%g]%s-LOOP[%g]: %s(%g:%g)\n",
        dmui, tracename, calci, msg, enteringVar, T[enteringVar]); T
}

        // ---------------------------------------------------------------------
        // Select leaving variable.
        // ---------------------------------------------------------------------
        leavingVar = 0
        Aj = Nj[,enteringVar]
        alpha = Bi * Aj
        if (existArtificial) {
            T = VARS[1,2::1+slacks]
            for (j=1; j<=cols(T); j++) {
                if (T[1,j] < 0 && alpha[j,1] != 0) {
                    leavingVar = j
                    break;
                }
            }
        }

        if (leavingVar == 0) {
            T = XB :/ (alpha :* (alpha :> 0))
            if (!allof(T, .)) {
                 minindex(T, 1, mi, w)
                 leavingVar = mi[1]
            }
        }
        lvj = 1+leavingVar


if (trace == "trace") {
    if (VARS[,lvj] < 0) T = XB :/ (alpha :* (alpha :> 0)) // for debug
    tempVal = (leavingVar == 0) ? .z : T[leavingVar]

    displayas("txt"); msg = "Select leaving variable."
    printf("\n[DMUi=%g]%s-LOOP[%g]: %s(%g:%g)\n",
        dmui, tracename, calci, msg, leavingVar, tempVal)
    display("XB | alpha | XB/alpha:"); XB,alpha,T
}

        // If leaving variable is not exit
        if (leavingVar == 0) {
            if (trace == "trace")
                display("Break: No more select leaving variable.")
            break
        }
        // If phase is 2, theta is not leaving. // FIXME: is correct ?
        if (phase == 2 && VARS[,lvj] == 1) {
            if (trace == "trace") display("Break: theta() is not leaving.")
            break
        }

        // ---------------------------------------------------------------------
        // reply calculatation result.
        // ---------------------------------------------------------------------
        T = M
        M[,lvj] = T[,evj]
        M[,evj] = T[,lvj]
        T = VARS
        VARS[,lvj] = T[,evj]
        VARS[,evj] = T[,lvj]
        LVi[leavingVar,1] = T[,evj]

        // Clear artificial variable at phase II.
        if (existArtificial) {
            if (VARS[,evj] < 0) {
                T = J(1, cols(VARS), 1)
                T[1,evj] = 0
                VARS = select(VARS, T)
                M = select(M, T)
                mcols = cols(M)
            }
            existArtificial = any(VARS[,2::1+slacks] :< 0)
        }

if (trace == "trace") {
    printf("\n[DMUi=%g]%s-LOOP[%g]: update tableau.[%g(%g) <--> %g(%g)]\n",
        dmui, tracename, calci, lvj, leavingVar, evj, enteringVar); M
    display("LVi: Entered variable's VARS index value."); LVi
    display("VARS: Variable's index."); VARS
}
        // ---------------------------------------------------------------------
        // Bi calculate...
        // If pivot value is zero, B's inverse skip
        // ---------------------------------------------------------------------
        alphaVal = alpha[leavingVar,]
        if (alphaVal == 0) continue

        // B's inverse revised.
        if (calci >= 30 && !allof((abs(b :- (B*XB)) :>= 1e-5), 0)) {
            Bi = edittozerotol(luinv(B), tol2)
        }
        else {
            T = alpha :/ (-alphaVal)
            T[leavingVar, ] = T[leavingVar, ] / (-alphaVal)
            Ei = I(slacks)
            Ei[,leavingVar] = T

            Bi = edittozerotol(Ei * Bi, tol2)
        }
    } //end of main for

    // Check the error.
    if (phase == 1 && !isExistSolution) {
        displayas("txt")
        printf("\n[DMUi=%g]%s-RESULT: CBBi * b = %g\n",
            dmui, tracename, xVal);
        displayas("txt")
        display("Entered index:"); VARS[1,2::1+rows(XB)]
        display("XB = Bi * b:"); XB

        _error(3498, "No Solution.:[DMUi=" + strofreal(dmui) + "]" + tracename)
    }

    return(XB)
}

function replacesubmat ( transmorphic matrix M,
                         real scalar row,
                         real scalar col,
                         transmorphic matrix T )
{
    M[|row,col\row + rows(T) - 1, col + cols(T) - 1|] = T
}

function _setup_dearslt_names(string scalar dearsltmat,
                              string scalar dmuinmat,
                              string scalar dmuoutmat )
{
    string matrix DMU_CS     // dmu in matrix column stripes
    string matrix DEARSLT_CS // dea result matrix column stripes
    string matrix DEARSLT_RS // dea result matrix row stripes
    real matrix M
    real scalar mcols, cnt

    M = st_matrix(dearsltmat)
    mcols = cols(M)

    DMU_CS = st_matrixcolstripe(dmuinmat)
    for (i = 1; i <= rows(DMU_CS); i++) {
        DMU_CS[i, 1] = "ref"
    }

    DEARSLT_CS = ("","rank"\"","theta")\DMU_CS\ // column join
        st_matrixrowstripe(dmuinmat)\st_matrixrowstripe(dmuoutmat)
    if (mcols - rows(DEARSLT_CS) > 0) {
        cnt = 0
        for (i = rows(DEARSLT_CS)+1 ; i <= mcols ; i++) {
            DEARSLT_CS = DEARSLT_CS \ ("slack", "slack_" + strofreal(++cnt))
        }
    }

    DEARSLT_RS = st_matrixcolstripe(dmuinmat)

    // setup dea result matrix row and column names
    st_matrixrowstripe(dearsltmat, DEARSLT_RS)
    st_matrixcolstripe(dearsltmat, DEARSLT_CS)
}

/**
 * deamat - dmucount x ( 1(theta) + dmu count + slcak(in, out) count)
 */
function _dmurank( string scalar deamat,
                   real scalar dmuincount,
                   real scalar dmuoutcount,
                   real scalar minrank,
                   real scalar tol )
{
    real matrix M
    real rowvector v, vv, retvec, slcaksum
    real scalar m, mm, row

    M = st_matrix(deamat)
    v = round(M[,1], tol)
    if (minrank) minindex(v, rows(v), i, w)
    else maxindex(v, rows(v), i, w)

    retvec = J(rows(v), 1, .)
    if (allof(w[,2], 1)) {
        retvec[i[1::rows(v)]] = (1::rows(v))
    }
    else {
        // rank correction for ties
        slcaksum = rowsum(M[|1,cols(M) - (dmuincount + dmuoutcount - 1)\.,.|])
        for (m = 1; m <= rows(w); m++) {
            if (w[m,2] >= 2) {
                vv = i[w[m,1]::(w[m,1] + w[m,2] - 1)]
                if (minrank) maxindex(slcaksum[vv], w[m,2], ii, ww)
                else minindex(slcaksum[vv], w[m,2], ii, ww)
                for (mm = 1; mm <= rows(ww); mm++) {
                    for (row = ww[mm,1]; row < ww[mm,1] + ww[mm,2]; row++) {
                        retvec[vv[ii[row]]] = w[m,1] + ww[mm,1] - 1
                    }
                }
            }
            else {
                retvec[i[w[m,1]]] = w[m,1] // row = w[m,1]
            }
        }
    }
    st_matrix("r(rank)", retvec)
}

function maxvecindex( string scalar vecname )
{
    real matrix A

    A = st_matrix(vecname)
    maxindex(A, 1, i, w)

    st_numscalar("r(maxval)", A[i[1]])
    st_numscalar("r(maxindex)", i[1])
    st_matrix("r(maxindexes)", i)
}

function minvecindex( string scalar vecname )
{
    real matrix A

    A = st_matrix(vecname)
    if (sum(A :< .) > 0) {
        minindex(A, 1, i, w)
        st_numscalar("r(minval)", A[i[1]])
        st_numscalar("r(minindex)", i[1])
        st_matrix("r(minindexes)", i)
    }
    // if overall missing value.
    else {
        st_numscalar("r(minval)", .)
        st_numscalar("r(minindex)", 0)
        st_matrix("r(minindexes)", 0)
    }
}

function _roundmat( string scalar matname, real scalar tol )
{
    real matrix A
    A = round(st_matrix(matname), tol)
    st_matrix(matname, A)
}

function _uniqrowmat( string scalar matname, string scalar varname )
{
    st_matrix(matname, sort(uniqrows(st_data(., varname)), 1))
}

function _file_exists( string scalar fn )
{
    st_numscalar("r(fileexists)", fileexists(fn))
}
end

// End of the MATA Definition Area ---------------------------------------------
